/*
 * Copyright (C) 2020 Michael Krause
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

import 'dart:async';

import 'package:array_object_adapter/array_object_adapter.dart';
import 'package:array_object_adapter_example/home_page_view_model.dart';
import 'package:array_object_adapter_example/model/comment_item.dart';
import 'package:array_object_adapter_example/model/post_item.dart';
import 'package:array_object_adapter_example/view/comment_item_presenter.dart';
import 'package:array_object_adapter_example/view/post_item_presenter.dart';
import 'package:flutter/material.dart';

///
class HomePage extends StatefulWidget {
  @override
  _HomePageState createState() => _HomePageState(viewModel: HomePageViewModel());
}

class _HomePageState extends State<HomePage> {
  _HomePageState({
    @required HomePageViewModel viewModel,
  }) : this._viewModel = viewModel;

  HomePageViewModel _viewModel;

  ArrayObjectAdapter _adapter;

  StreamSubscription streamSubscription;

  @override
  void initState() {
    super.initState();

    final classPresenterSelector = ClassPresenterSelector()
      ..addClassPresenter(PostItem, PostItemPresenter())
      ..addClassPresenter(CommentItem, CommentItemPresenter());
    _adapter = ArrayObjectAdapter(presenterSelector: classPresenterSelector);
    streamSubscription = _viewModel.content.listen((result) {
      _adapter.setItems(result);
    });
    _viewModel.downloadContent();
  }

  @override
  void dispose() {
    streamSubscription?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white.withAlpha(225),
        appBar: AppBar(title: Text("Related Posts")),
        body: Padding(
          padding: const EdgeInsets.all(16.0),
          child: StreamBuilder<Object>(
              stream: _adapter, // notified on data set changed
              builder: (context, snapshot) {
                return ListView.separated(
                  itemCount: _adapter.itemCount(),
                  itemBuilder: (BuildContext context, int index) {
                    final item = _adapter.get(index);
                    final presenter = _adapter.getPresenter(item);
                    return presenter.createView(context, item);
                  },
                  separatorBuilder: (BuildContext context, int index) {
                    final nextItem = _adapter.get(index + 1);
                    if (nextItem is PostItem) {
                      return SizedBox(height: 48);
                    }
                    return Container(height: 4,);
                  },
                );
              }),
        ));
  }
}
